// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: CheckExpr
// Author: Hassan Siddiqui
//
// Description: 
//
// A simple tool to check whether a selection expression
// contains any entries of type "true".
//
// Modifications: See ChangeLog
// 
// $Id: CheckExpr.cc,v 1.2 2004/08/17 14:02:56 rsaxton Exp $


/* SAS classes */
#include "Tracer.h"
#include "errstr.h"

/* Local classes */
#include "CheckExpr.h"


bool CheckExpr::check(const string & instr)
{
  PROC;

  string str=instr;

  string::size_type left = 0;
  string::size_type right;

  bool done=false;

  while (!done)
    {
      string::size_type right1 = str.find("||");
      string::size_type right2 = str.find("&&");
      
      if (right2 < right1 || right1==string::npos)
	{
	  right=right2;
	}
      else
	{
	  right=right1;
	}

      errstr 
	<< " left="  <<left
	<< " right="  <<right
	<< " right1="  <<right1
	<< " right2="  <<right2
	<< Sas::message(Msg::App, Msg::Noisy);
      
      string token;
      
      if (right == string::npos)
	{
	  token=str;
	  done=true;
	}
      else
	{
	  
	  token=str.substr(left,right);
	  str = str.substr(right+2);
	}
      
      errstr 
	<< " token=["<< token.c_str()<<"]"
	<< " reminder=["<< str.c_str()<<"]"
	<< Sas::message(Msg::App, Msg::Noisy);

      if ( token.find("true") == string::npos )
	{
	  errstr 
	    << "The token ["<< token << "] is not true."
	    << Sas::message(Msg::App, Msg::Noisy);

	  ALTRETURNV(false);
	}
    }

  RETURN true;
}

bool CheckExpr::hasMask(const string & instr)
{
  PROC;

  string str=instr;

	  errstr 
	    << "hasMask input:" <<str<<" "<<str.find("mask")
	    << Sas::message(Msg::App, Msg::Noisy);
  
  if (str.find("mask") != string::npos || 
      str.find("MASK") != string::npos) {
        RETURN true;
  } else {
        RETURN false;
  }
}

