// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: OffCCDPixelIterator
// Author: Hassan Siddiqui
//
// Description:
//
// Class representation of an iterator that handles DAL-based maps
// in table format.
//
// Modifications: See ChangeLog
//
// $Id: OffCCDPixelIterator.h,v 1.7 2014/04/03 13:37:51 rsaxton Exp $

#ifndef TABLEPIXELITERATOR_H
#define TABLEPIXELITERATOR_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"
#include "Region.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "PixelIterator.h"
#include "WCSConvertor.h"
#include "PixelElement.h"
#include "RegionExtent.h"
#include "PositionUtils.h"


class OffCCDPixelIterator : public PixelIterator
{
public:

  /* Constructors */
  OffCCDPixelIterator(PixelIterator * BSPixelIt,
                   const PositionUtils * const putils, unsigned long flags,
                   bool ignoreoutoffov, unsigned long ccdmask);

  /* Destructors */
  ~OffCCDPixelIterator();

  /* Iterator methods */
  void first ();
  void next  ();
  bool isDone();
  PixelElement currentItem();

  const WCSConvertor * wcsconvertor() const;

  unsigned  xbins() const;

  /* Is pixel on a CCD ? */
  bool onChip(const PositionUtils * const putils, unsigned long flags,
           bool ignoreoutoffov, unsigned long ccdmask,
           CCDwindow *ccdwindow, unsigned cent_ccd,
           unsigned minx, unsigned miny, unsigned maxx, unsigned maxy,
           double detx, double dety); 

  /* The total number of elements off CCD */
  unsigned size()   const;

  /* The total number of detector pixels in input region */
  unsigned totalPixels()   const;

private:

  bool _firstcalled;

  unsigned long _index;
  unsigned long _offccdelements;
  unsigned long _totalelements;

  const PositionUtils *  _putils;
  vector<double> _detPixWidths;

  WCSConvertor * _wcsconv;

  /* Camera coordinates of pixels lying within source area and on chip gap */
  vector<PixelElement> _OffCCDPixel;

  /* The width of elements in the grid used to determine points off
     the CCD (in detector pixels) */
  float _gridWidth;
};

#endif
