// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: PointEeCorrector
// Author: Hassan Siddiqui
//
// Description: 
//
// Point source implementation of an Encircled Energy Corrector class
//
// Modifications: See ChangeLog
// 
// $Id: PointEeCorrector.cc,v 1.7 2012/03/29 13:14:00 rsaxton Exp $

/* General */

/* SAS-specific */
#include "Tracer.h"

/* Local */
#include "NumericalRange.h"
#include "PointEeCorrector.h"
#include "PixelIterator.h"
#include "ArfDataServer.h"
#include "CalServer.h"

/* Constructor */
PointEeCorrector::PointEeCorrector(unsigned eefactor):
  IterEeCorrector(eefactor)
{
  PROC;
  ENDPROC;
}

/* Destructor */
PointEeCorrector::~PointEeCorrector()
{
  PROC;
  ENDPROC;
}

double PointEeCorrector::eeFraction( double energy  ) const
{
  PROC;

  
  double fracenclosed   = 0;
  double contrib = 0;

      errstr << "PointEeCorrector: calles"
	     << Sas::message(Msg::App, Msg::Verbose);
      
 /* Get the PSF function and an iterator for its integration from the 
    ARF data server */

  PixelIterator * pxitr = arfDataServer->PsfPixelIterator();
  PsfModel * psf = arfDataServer->psf();

 /* Set the energy for this PSF model */
  psf->energy(energy);

 /* Integrate the PSF contribution */
  if (calServer.state()->accuracyLevel()->get() == HIGH)
  {
  FOREACH(pxitr)
    {
      PixelElement pixel = pxitr->currentItem();
      
         contrib = psf->int_psf( pixel.xCentre(),
			      pixel.yCentre(),
			      pixel.xWidth(),
			      pixel.yWidth());
      
      errstr << "PointEeCorrector: Pixel details  " << pixel << endl
	     << " PSF contribution " << contrib << endl
	     << Sas::message(Msg::App, Msg::Noisy);
      
      fracenclosed += contrib;
    }
  } 

  double fracenclosed2 = 0.0;
  if (calServer.state()->accuracyLevel()->get() == ELLBETA ||
      calServer.state()->accuracyLevel()->get() == EXTENDED ) {
     // COmment out the old call
 //    fracenclosed = psf->rad_psf();  // RDS - comment out 
     fracenclosed2 = psf->enclosed_psf(pxitr);
  }

  errstr << "PointEeCorrector: Fraction of psf enclosed " 
	 << energy<<" "<<fracenclosed << " "<<fracenclosed2<<endl
	 << Sas::message(Msg::App, Msg::Verbose); 

  RETURN fracenclosed2;
}
  
